package edu.harvard.mgh.purcell.gplink.base_form;

import java.awt.Component;
import java.io.*;

//import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.harvard.mgh.purcell.gplink.data.*;

/**
 * An extention of JPanel that gets the output root from the user.
 * @author ktoddbrown
 */
@SuppressWarnings("serial")
public class OutFilePanel extends JPanel {
	
	private JTextField outRoot;
	private Project data;
	
	private Forms parent;
	//public boolean validInput;
	
	/**
	 * Update the message that tells the user if the current
	 * output root is valid or not
	 */
	DocumentListener messageUpdate = new DocumentListener(){
		public void insertUpdate(DocumentEvent e) {
			feedback();
			parent.isValidCommand();
			//validInput = isValidInput();
		}
		public void removeUpdate(DocumentEvent e) {
			feedback();
			parent.isValidCommand();
			//validInput = isValidInput();
		}
		public void changedUpdate(DocumentEvent e) {
			feedback();
			parent.isValidCommand();
			//validInput = isValidInput();
		}
	};
	
	public String getCommand(){
		String ans = outRoot.getText();
		
		if(isValidInput() == false){
			JOptionPane.showMessageDialog(parent.myFrame,
				"Error trying to get output root from the current form. " +
				"You should never see this message.",
    			"gPLINK error",
    			JOptionPane.ERROR_MESSAGE);
			return null;
		}
		//we need the remote commands to have the full name
		//...otherwise all outputs go to the users home directory
		if(data.isRemote(false))
			return "--out " + GeneralUtil.quoteFilename(data.getRemoteFile() + "/" +  ans);
		else
			return "--out " + GeneralUtil.quoteFilename(ans);
	}
	
	/**
	 * check if a log file exists and is non-empty
	 * @return returns if the output log file exists and is non-empty
	 */
	public boolean isValidInput(){
		//check the quick one first
		if(outRoot.getText().matches("^[\\w\\-]+$") == false)
			return false;
		
		String potentialLog = outRoot.getText() + ".log";
		
		for(String file:parent.allFiles){
			if(potentialLog.equals(GeneralUtil.fileName(file)))
				return false;
		}
		
		return true;
		
	}
	
	/**
	 * set message to reflect the validity of a output root file
	 */
	private void feedback(){
		if(isValidInput())
			setBorder(new TitledBorder("Output file root: Valid fileroot"));
		else
			setBorder(new TitledBorder("Output file root: Invalid fileroot"));
		
	}
	
	/**
	 * Create a panel to get the output file data from the user
	 * @param p the project this panel is tied to
	 */
	public OutFilePanel(Project p, Forms givenF){
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(new TitledBorder("Output file root: Invalid fileroot"));
		
		data = p;
		parent = givenF;
		
		
		//this is the text field that user will input the output root to
		outRoot = new JTextField(40);
		//we want the user to be able to input the root
		outRoot.setEditable(true);
		//this let's us give feed back on the chosen root
		outRoot.getDocument().addDocumentListener(messageUpdate);
		
		
		JPanel filePanel = new JPanel();
		filePanel.setLayout(new BoxLayout(filePanel, BoxLayout.Y_AXIS));
		filePanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel rootname = new JLabel(data.getRoot().getAbsolutePath() 
				+ File.separator);
		filePanel.add(rootname);
		filePanel.add(outRoot);
		this.add(filePanel);
		
	}
}
