package edu.harvard.mgh.purcell.gplink.forms;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.ButtonGroup;

import edu.harvard.mgh.purcell.gplink.base_form.*;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;

@SuppressWarnings("serial")
public class GlmForm extends Forms {

	//buttons that allow the user to select the --freq and
	//...--count commands
	Forms.myRadioButton freqButton, genoButton;
	//allows the user to specify an optional --within
	Forms.myCheckBox covarButton;
	//picks the --within file
	Forms.myJTextField covarField;
	//button pick the file associated with --within
	Forms.pickAFileButton pickCovar;

		
	public GlmForm(MainFrame mf) {
		super(mf, "Linear/logistic regression tests");	
	}

	@Override
	protected void createBody() {
		//put the --freq and --count in a buton group so that
		//...the user can only use one or the other and is forced
		//...to select atleast one to excicute the form.
		ButtonGroup pickOne = new ButtonGroup();
		pickOne.add(freqButton);
		pickOne.add(genoButton);
		
		//set the layout for the form
		body.setLayout(new GridBagLayout());
		//declare and initalze the constraits for this layout
		GridBagConstraints c = new GridBagConstraints();
		//expand all the components in this layout to fill all
		//...avaliable horizontal space
		c.fill = GridBagConstraints.BOTH;
		//add the frequency button to the layout with the given
		//...constraints
		body.add(freqButton,c);
		//move down before adding the next one
		c.gridy = 1;
		body.add(genoButton,c);
		//move down again
		c.gridy = 2;
		body.add(covarButton, c);
		//let the next component resize its self to fill all avaliable
		//...space
		c.weightx = 1;
		c.gridx = 1;
		body.add(covarField, c);
		//we don't want the next component to resize
		c.weightx = 0;
		c.gridx = 2;
		
		body.add(pickCovar, c);
		
	}

	@Override
	protected void initalize() {
		covarField = new myJTextField(10);
		
		pickCovar = new pickAFileButton(".cov", "Covariate File", covarField);
		pickCovar.setEnabled(false);
		
		freqButton = new myRadioButton("Linear/logistic regression (--linear)");
		freqButton.setSelected(true);
		genoButton = new myRadioButton("2df genotypic model (--linear --genotypic)");
		covarButton = new myCheckBox("Covariate file (--covar)", 
				covarField, pickCovar);

	}

	@Override
	protected boolean isBodyValid() {
		if( covarButton.isSelected() 
				&& covarField.getText().length() == 0){
//			information.setText("When --covar is selected you must specify a file.");
			return false;
		}
		else 
			return true;
	}

	@Override
	protected String processBody() {
		String ans = "--linear";
		
		if(genoButton.isSelected())
			ans += " --genotypic";
	
		if(covarButton.isSelected())
			//since within takes a filename we need to throw
			//...quotes around it if we are in a window's os
			ans += " --covar " + GeneralUtil.quoteFilename(covarField.getText());

		
		return ans;
	}

}
