package edu.harvard.mgh.purcell.gplink.mainGUI;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import java.awt.*;
import java.awt.event.*;
//import java.io.File;

import javax.swing.ImageIcon;

import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
import edu.harvard.mgh.purcell.gplink.data.AutoUpdater;

/**
 * OpView provides a JPanel that contains an actionable list of operations.
 * As operation are selected in this tree the log viewer is update to reflect
 * the current operation selection. There are also certain popup menu items
 * defined that allow the user to interact with and change the project 
 * annotations as well as launch external programs to interact with the
 * in/out files in the project.
 * 
 * @author Kathe Todd-Brown
 * 
 */
@SuppressWarnings("serial")
public class OpView extends JPanel {

	/**
	 * frame that this panel is displayed in
	 */
	private MainFrame frame;

	/**
	 * The operation tree
	 */
	private JTree viewTree;

	/**
	 * The tree model that allows us to dynamically modify the
	 * tree
	 */
	private DefaultTreeModel updateableTree;

	/**
	 * The last operation in the list, this allows us to scroll
	 * to the last entry in theory. This should probably be checked
	 * at some point in the future
	 */
	private DefaultMutableTreeNode lastNode;
	
	/**
	 * Dynamically update the tree in the panel
	 */
	public void updateView() {
		//System.out.println("updateView@OpView.java: Entering function.");
		DefaultMutableTreeNode top;
		
		// Check to make sure that there is a valid project
		if (frame.data.getRoot() != null) {
			//reset the project name
			top = new DefaultMutableTreeNode(frame.data
					.getProjectName());
			//System.out.println("flag1");
			//clear everything out
			//top.removeAllChildren();
			//System.out.println("flag2");
			//populate the tree
			lastNode = createTreeNodes(top);
			//System.out.println("flag3");
			//set the dynamic tree to the root node
		}else{
			top = new DefaultMutableTreeNode("No root file selected");
			lastNode = null;
		}
		
		updateableTree.setRoot(top);
		updateableTree.nodeChanged(top);
		//System.out.println("flag4");
		//make the changes visible
		updateableTree.reload();
		//System.out.println("flag5");
		if(lastNode != null)
			viewTree.scrollPathToVisible(new TreePath(lastNode.getPath()));
		
		//System.out.println("updateView@OpView.java: Exiting function.");
	}

	/**
	 * Populate the tree with the data from the project.
	 * @param t a DefaultMutableTreeNode that holds the project name
	 * @return the TreeNode that holds the project name but now populated
	 * with the rest of the tree structure
	 */
	@SuppressWarnings("static-access")
	public DefaultMutableTreeNode createTreeNodes(DefaultMutableTreeNode t) {
		//a node containing the operations data
		DefaultMutableTreeNode opNode = null;
		//a node containing the file data
		DefaultMutableTreeNode fileNode;
		//a node containing all the input file nodes as children
		DefaultMutableTreeNode inf;
		//a node containing all the output files as children
		DefaultMutableTreeNode outf;
		//a node contianing the command data
		DefaultMutableTreeNode command;

		//go through each of the operations in the project
		for (String s : frame.data.getAllOpName()) {
			//populate this operation node with the name and discription of
			//...the operation
			opNode = new DefaultMutableTreeNode(s + ": " 
					+ frame.data.getOpDiscription(s));
			//populate the command
			command = new DefaultMutableTreeNode(frame.data.getOpCommand(s));
			//reset the infile nodes
			inf = new DefaultMutableTreeNode("Input files:");
			//reset the outfile nodes
			outf = new DefaultMutableTreeNode("Output files:");
			//get the input files
			String[][] infileInfo = frame.data.getOpFiles(s, frame.data.INFILE_KEY);
			//if the file information is valid
			//System.out.println("createTreeNodes@OpView get the input files");
			if(infileInfo != null 
					&& infileInfo.length == 2
					&& infileInfo[0] != null 
					&& infileInfo[0].length >= 1
					&& infileInfo[0][0] != null){
				//go through and add each file and discription
				for (int i = 0; i < infileInfo[1].length; i++) {
					// add the file name
					fileNode = new DefaultMutableTreeNode(infileInfo[0][i]);
					String globalDisc = frame.data.getFolderFileNode(
							GeneralUtil.fileName(infileInfo[0][i]));
					// add the file discription if it's not empty whitespace
					if(!infileInfo[1][i].matches("^\\s*$") 
							|| !globalDisc.equals("")){
						fileNode.add(new DefaultMutableTreeNode("Description: "
							+ globalDisc + "; "
							+ infileInfo[1][i]));
					}
					//add it to the input file node
					inf.add(fileNode);
				}
			}
			
			//get the output files
			String [][] outfileInfo = frame.data.getOpFiles(s, frame.data.OUTFILE_KEY);
			//System.out.println("createTreeNodes@OpView get the output files");
			//if the file information is valid
			if(outfileInfo != null 
					&& outfileInfo.length == 2
					&& outfileInfo[0] != null 
					&& outfileInfo[0].length >= 1
					&& outfileInfo[0][0] != null){
				for (int i = 0; i < outfileInfo[1].length; i++) {
					//don't add the file if it is the operation status flag, user
					//...doesn't need to see this
					if(!outfileInfo[0][i].endsWith(frame.data.OP_STATUS_EXT)){
						// add the file name
						fileNode = new DefaultMutableTreeNode(outfileInfo[0][i]);
						String globalDisc = frame.data.getFolderFileNode(
								GeneralUtil.fileName(outfileInfo[0][i]));
						// add the file discription if it is not whitespace
						if(!outfileInfo[1][i].matches("^\\s*$")
								|| !globalDisc.equals("")){
							fileNode.add(new DefaultMutableTreeNode("Description: "
									+ globalDisc + "; "
									+ outfileInfo[1][i]));
						}
						//add it to the output file node
						outf.add(fileNode);
					}
				}
			}
			//add the command to the operation
			opNode.add(command);
			//add the input files
			opNode.add(inf);
			//add the output files
			opNode.add(outf);
			//add this operation to the project node
			t.add(opNode);
		}
		//return the last operation that was added to the project node
		return opNode;

	}

	/**
	 * This TreeSelectionListener updates the log viewer to reflect
	 * a switch in operation selection. Note that if nodes within
	 * an operation are selected then the log viewer catches this and
	 * does nothing.
	 */
	private TreeSelectionListener changelog = new TreeSelectionListener() {
		
		public void valueChanged(TreeSelectionEvent e) {
			//double check that we have a valid path selection
			if (OpView.this.viewTree.getSelectionModel().getSelectionPaths() != null) {
				//get the operation name associated with the selected node
				String opName = getOpName()+".log";
				//update the log viewer
				frame.logViewer.update(opName, false);
			}
		}
	};

	/**
	 * Listen for a tree selection that will activate the popup menu
	 * to manipulate the project annotation and open in/out files.
	 */
	private MouseListener treeEvents = new MouseListener() {
		public void mousePressed(MouseEvent e) {
			checkEvent(e);
		}

		public void mouseReleased(MouseEvent e) {
			checkEvent(e);
		}

		public void mouseClicked(MouseEvent e) {
			return;
		}

		public void mouseEntered(MouseEvent e) {
			return;
		}

		public void mouseExited(MouseEvent e) {
			return;
		}

		/**
		 * If this is a popup event then create a popup menu.
		 * @param e the mouse event that triggered the tree selection
		 */
		private void checkEvent(MouseEvent e) {
			//check for popup trigger
			if (e.isPopupTrigger()) {
				//check validity of the selection
				if (OpView.this.viewTree.getSelectionModel()
						.getSelectionPaths() != null) {
					//get the file if it is selected
					String [] filename = new String [1];
					if(OpView.this.viewTree
							.getSelectionPaths()[0].getPathCount() >= 4){
						filename[0] = OpView.this.viewTree.getSelectionPaths()[0].getPath()[3].toString();
					} else {
						filename = null;
					}
					//create a new popup from this path
					OpTreePopup p = new OpTreePopup(frame, frame.data, new String []{ getOpName()},
							filename);
					//show the popup
					p.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		}
	};

	/**
	 * Move the selected node up the tree.
	 */
	private Action swapOpDown = new AbstractAction(){
		public void actionPerformed(ActionEvent e) {
			//get the current selected path
			TreePath current = viewTree.getSelectionPath();
			//get the current row
			int currentrow = viewTree.getSelectionRows()[0];
			//get the current operation node
			DefaultMutableTreeNode currentOp = 
				(DefaultMutableTreeNode)((current.getPath())[1]);
			//get the following operation node
			DefaultMutableTreeNode next = currentOp.getNextSibling();
			//check to see if you are at the end of the tree
			if(next != null){
				//swap the operation in the underlaying data structure
				//Node: we change the underlaying data structure to perserve the
				//...ordering in exporting the data and between sessions
				frame.data.swapOp(currentOp.toString().split(":")[0], 
						next.toString().split(":")[0]);
				//redraw the whole thing. this doesn't take much time
				OpView.this.updateView();
				//select the node we just moved
				viewTree.addSelectionRow(currentrow+1);
			}
		}
	};
	
	/**
	 * Move the selected node down the tree.
	 */
	private Action swapOpUp = new AbstractAction(){
		public void actionPerformed(ActionEvent e) {
			//get the current selected path
			TreePath current = viewTree.getSelectionPath();
			//get the current row
			int currentrow = viewTree.getSelectionRows()[0];
			//get the current operation node
			DefaultMutableTreeNode currentOp = 
				(DefaultMutableTreeNode)((current.getPath())[1]);
			//get the previous operation node
			DefaultMutableTreeNode previous = currentOp.getPreviousSibling();
			//check to see if we are in the start of the tree
			if(previous != null){
				//swap the operation in the underlaying data structure
				//Node: we change the underlaying data structure to perserve the
				//...ordering in exporting the data and between sessions
				frame.data.swapOp(currentOp.toString().split(":")[0], 
						previous.toString().split(":")[0]);
				//redraw the whole thing. this does not take much time
				OpView.this.updateView();
				//select the node we just moved
				viewTree.addSelectionRow(currentrow-1);
			}
		}
	};
	
	/**
	 * Pull the operation associated with the first selected node.
	 * @return a string that is the operation name associated
	 * with the first selected node
	 */
	private String getOpName() {
		//get the string stored in the operation node
		String base = ((OpView.this.viewTree.getSelectionModel()
				.getSelectionPaths()[0]
		        .getPath()[1]).toString());
		//seperate the operation name from the discription
		base = processOpName(base);
		//return the operation name
		return base;
	}
	
	/**
	 * Pull the operation name from the text in the operation
	 * node.
	 * @param s a string from the operaiton node in the form "name: discription"
	 * @return a string containing the name of the operaiton
	 */
	private String processOpName(String s){
		//the operation name and the discription are seperated by a :
		//...so split the string at the :
		s = s.split(":")[0];
		//if we have an html flag
		if(s.contains("<html>"))
			//cut it off
			s = s.substring(new String ("<html>").length(), s.length());
		//return the operation name
		return s;
	}
	
	
	/**
	 * Create an operations view of a project
	 * 
	 * @param givenFrame the MainFrame that this operation viewer is
	 * attached to.
	 */
	public OpView(MainFrame givenFrame) {
		super();

		frame = givenFrame;
		//set the only element to fill the entire panel
		setLayout(new BorderLayout());
		//put a title on the viewer
		setBorder(new TitledBorder("Operations viewer"));

		////////////////////////////////////////////////////////
		//*******Create the tree********************************
		////////////////////////////////////////////////////////
		
		//create the top node that will hold the project name but never be shown
		DefaultMutableTreeNode top;
		//check to see if the project is set
		if (frame.data.getRoot() == null) {
			//if it isn't throw on a default that we hope will never
			//...occure
			top = new DefaultMutableTreeNode("No root file selected");
		} else {
			//otherwise pull the project name
			top = new DefaultMutableTreeNode(frame.data.getProjectName());
			//and populate the tree
			createTreeNodes(top);
		}
		//inalize the updateable tree based on this project node
		updateableTree = new DefaultTreeModel(top);
		
		/////////////////////////////////////////////////////////
		//***************Set the tree behavior*******************
		/////////////////////////////////////////////////////////
		
		//intalize the tree based on this dynamic model
		viewTree = new JTree(updateableTree);
		//scroll to the last node
		if(lastNode != null)
			viewTree.scrollPathToVisible(new TreePath(lastNode.getPath()));
		//only let the user select one node at a time
		viewTree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		//don't let the user see the root since it's not 
		//...really interesting
		viewTree.setRootVisible(false);
		//show useful icons
		viewTree.setShowsRootHandles(true);
		//swap the log viewer to reflect the currently selected node
		viewTree.addTreeSelectionListener(changelog);
		//listen for popup events on the tree
		viewTree.addMouseListener(treeEvents);
		//put the tree in a scroll pane
		add(new JScrollPane(viewTree));
		
		////////////////////////////////////////////////////////////
		//Use our renderer's to inform the user of the status of the operations
		////////////////////////////////////////////////////////////
		
		//Set the icon for leaf nodes.
        ImageIcon sucessIcon = createImageIcon("images/gp1.gif");
        ImageIcon failIcon = createImageIcon("images/gp2.gif");
        ImageIcon runningIcon = createImageIcon("images/gp3.gif");
        ImageIcon fileInfoIcon = createImageIcon("images/fileInfoIcon.GIF");
        //if these all exist then use our renderer
        if (sucessIcon != null && failIcon != null && runningIcon != null) {
            viewTree.setCellRenderer(
            		new MyRenderer(sucessIcon, failIcon, runningIcon, fileInfoIcon));
        }
	
		////////////////////////////////////////////////
		//***Move the operations up and down the tree***
		////////////////////////////////////////////////
		viewTree.getInputMap().put(KeyStroke.getKeyStroke
				("control 1"), "move up");
		viewTree.getActionMap().put("move up", swapOpUp);
		viewTree.getInputMap().put(KeyStroke.getKeyStroke
				("control 2"), "move down");
		viewTree.getActionMap().put("move down", swapOpDown);
		
	}

	
	/** 
	 * Intelligently create an image URL
	 * @return an ImageIcon, or null if the path was invalid.
	 */
    protected ImageIcon createImageIcon(String path) {
    	java.net.URL imgURL = OpView.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }
    
	/**
	 * Create a renderer to reflect that status of each operation 
	 * (completed, failed or running).
	 * @author Kathe Todd-Brown
	 *
	 */
	private class MyRenderer extends DefaultTreeCellRenderer {
        Icon sucessIcon;
        Icon failIcon;
        Icon runningIcon;
        Icon fileInfoIcon;
        
        /**
         * Create a new renderer
         * @param sicon the icon that flags the sucess status
         * @param ficon the icon that flags the failed status
         * @param ricon the icon that flags the running status
         */
        public MyRenderer(Icon sicon, Icon ficon, Icon ricon,
        		Icon fileIcon) {
        	sucessIcon = sicon;
            failIcon = ficon;
            runningIcon = ricon;
            fileInfoIcon = fileIcon;
        }

       /**
        * Set the icon of each node.
        */
        public Component getTreeCellRendererComponent(  JTree tree, 
        		Object value, boolean sel, boolean expanded, boolean leaf, 
        		int row, boolean hasFocus) {

            super.getTreeCellRendererComponent( tree, value, sel, expanded,
            		leaf, row, hasFocus);
            
            if (isRunningOp(value)) {
            	setIcon(runningIcon);
            } else if (isCompleteOp(value)) {
            	 setIcon(sucessIcon);
            } else if (isFailedOp(value)) {
            	setIcon(failIcon);
            } else if(leaf == false 
            		&& isFile(value)){
            	setIcon(fileInfoIcon);
            }
            return this;
        }

        private boolean isFile(Object value){
        	return (value != null 
        			&& !value.toString().endsWith(":"));
        }
        
        private boolean isRunningOp(Object value) {
        	if(frame.data.gplinkLog != null
        			&& frame.data.gplinkLog.opStatis != null
        			&& frame.data.gplinkLog.opStatis
        				.get(processOpName(value.toString())) != null
        			&& frame.data.gplinkLog.opStatis
        				.get(processOpName(value.toString()))
        					.equals(AutoUpdater.RUNNING))
        		return true;
            return false;
        }
        
        protected boolean isCompleteOp(Object value) {
        	if(frame.data.gplinkLog != null
        			&& frame.data.gplinkLog.opStatis != null
        			&& frame.data.gplinkLog.opStatis
        				.get(processOpName(value.toString())) != null
        			&& frame.data.gplinkLog.opStatis
        				.get(processOpName(value.toString()))
        					.equals(AutoUpdater.COMPLETE))            
            	return true;
            return false;
        }
        
        protected boolean isFailedOp(Object value) {
        	if(frame.data.gplinkLog != null
        			&& frame.data.gplinkLog.opStatis != null
        			&& frame.data.gplinkLog.opStatis
        				.get(processOpName(value.toString())) != null
        			&& frame.data.gplinkLog.opStatis
        				.get(processOpName(value.toString()))
        					.equals(AutoUpdater.ERROR))           
            	return true;
            return false;
        }
    }

}
