package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
import edu.harvard.mgh.purcell.gplink.data.Project;

@SuppressWarnings("serial")
public class OpenHapDialog extends JDialog {

	static String [] pedSuffex = new String [] {".ped"};
	static String [] infoSuffex = new String [] {".info"};
	static String [] mapSuffex = new String [] {".map", ".bim"};
	static String [] nonSNPSuffex = new String [] {".mds",
		".imendel", ".fmendel", ".mendel", ".imiss", ".het"};
	static String [] SNPSuffex = new String [] {".adjusted",
		 ".assoc", ".assoc.mperm", 
		".assoc.perm", ".cmh", ".cmh2", ".frq", ".hwe",
		".lmiss", ".missing", ".missing.hap", 
		".linear", ".logistic", ".genotypic", ".model", 
		".qassoc", ".qassoc.gxe", ".tdt", ".tdt.mperm", 
		".tdt.perm"};
	
	MainFrame frame;
	JTextField mapFile;
	JTextField SNPFile;
	JTextField nonSNPFile;
	JTextField pedFile;
	JTextField infoFile;
	
	GeneralUtil.pickAFileButton pedButton;
	GeneralUtil.pickAFileButton mapButton;
	GeneralUtil.pickAFileButton SNPButton;
	GeneralUtil.pickAFileButton nonSNPButton;
	GeneralUtil.pickAFileButton infoButton;
	
	JRadioButton pedInfoRB;
	JRadioButton nonSNPRB;
	JRadioButton SNPRB;
	
	JButton ok;
	JButton cancel;
	
	private ActionListener dispose = new ActionListener(){

		public void actionPerformed(ActionEvent arg0) {
			OpenHapDialog.this.dispose();
		}
		
	};
	
	/**
	 * open in haploview
	 */
	private ActionListener openHaplo = new ActionListener(){

		public void actionPerformed(ActionEvent e) {
			
			/*
			 * *************************************************************
			 * Deal with ped/info combo since it has a seperate command line
			 * *************************************************************
			 */

			if(pedInfoRB.isSelected()){	
				String [] hapOptions = GeneralUtil.stripAndSplit(
						frame.data.getHaploviewAppend());
				String [] command = new String [ 7 +
				         hapOptions.length];
				command[0] = " java";
				command[1] = " -jar";
				command[2] = frame.data.getHaploviewPath();
				command[3] = " -p ";
				command[4] = pedFile.getText();
				command[5] = " -i ";
				command[6] = infoFile.getText();
				
				//append the projects haploview
				if(!hapOptions[0].equals(null)){
					for(int i = 0; i < hapOptions.length; i ++){
						command[i+7] = hapOptions[i];
					}
				}
				
				//send it off to the os
				try{	
					@SuppressWarnings("unused")
					Process haplo = Runtime.getRuntime().exec(command);
					
				} catch (IOException e1) {
		        	new ErrorDialog(frame,
		    	        	"Error trying to open the following files " +
    	        			"in haploview " +
    	        			"[ "+pedFile.getText()+" ], [ "+infoFile.getText()+" ].\n" +
    	        			"openHaploview:TreePopup.java");
		        }
				//we are done!
				OpenHapDialog.this.dispose();
		        return;
			}
					
			/*
			 * ****************************************** 
			 * Deal with non-SNP files in Haploview
			 * ******************************************
			 */
			

			if(nonSNPRB.isSelected())		
			{
			String filename = nonSNPFile.getText();
			if(new File(filename).exists() == false )
			{
				if(frame.data.isRemote(false))
					new ErrorDialog(frame, 
						"[ " + filename + " ] does not exist, " +
								"can not open Haploview.\n" +
						"Possibly not downloaded from remote server.\n" +
						"openHaplo:TreePopup.java");
					
				else
					new ErrorDialog(frame, 
						"[ " + filename + " ] does not exist, " +
								"can not open Haploview.\n" +
						"openHaplo:TreePopup.java");
				OpenHapDialog.this.dispose();
				return;
			}
						
			String [] hapOptions = GeneralUtil.stripAndSplit(
					frame.data.getHaploviewAppend());
			String [] command = new String [ 6 +
			         hapOptions.length];
			command[0] = "java";
			command[1] = "-jar";
			command[2] = frame.data.getHaploviewPath();
			command[3] = "-plink";
			command[4] = filename;
			command[5] = "-nonSNP";
			//add any extra's to the haploview command
			if(!hapOptions[0].equals(null)){
				for(int i = 0; i < hapOptions.length; i ++){
					command[i+6] = hapOptions[i];
				}
			}
			try{
				@SuppressWarnings("unused") 
				Process haplo = Runtime.getRuntime().exec(command);
			} catch (IOException e1) {
	        	new ErrorDialog(frame,
	        			"Error trying to open the following files in Haploview " +
	        			"[ " + filename + "]. \n" +
	        			"openHaplo:TreePopup.java");
	        }
			OpenHapDialog.this.dispose();
			return;
			}
			
			/*
			 * ****************************************** 
			 * Deal with remaining files (SNP results)
			 * ******************************************
			 */
			
			if(SNPRB.isSelected()){
				String filename = SNPFile.getText();
				String infile = mapFile.getText();
				if(new File(filename).exists() == false 
						|| new File(infile).exists() == false)
				{
					if(frame.data.isRemote(false))
						new ErrorDialog(frame, 
							"[ " + filename + " ] and/or [ "+ infile + " ] does not exist, " +
									"can not open haploview.\n" +
							"Possibly not downloaded from remote server.\n" +
							"openHaplo:TreePopup.java");
						
					else
						new ErrorDialog(frame, 
							"[ " + filename + " ] and/or [ "+ infile + " ] does not exist, " +
									"can not open haploview.\n" +
							"openHaplo:TreePopup.java");
					OpenHapDialog.this.dispose();
					return;
				}
				
				String [] hapOptions = GeneralUtil.stripAndSplit(
						frame.data.getHaploviewAppend());
				String [] command = new String [ 7 +
				         hapOptions.length];
				command[0] = "java";
				command[1] = "-jar";
				command[2] = frame.data.getHaploviewPath();
				command[3] = "-plink";
				command[4] = filename;
				command[5] = "-map"; 
				command[6] = infile;
				//add any extra's to the haploview command
				if(!hapOptions[0].equals(null)){
					for(int i = 0; i < hapOptions.length; i ++){
						command[i+7] = hapOptions[i];
					}
				}
				try{
					@SuppressWarnings("unused") 
					Process haplo = Runtime.getRuntime().exec(command);
				} catch (IOException e1) {
		        	new ErrorDialog(frame,
		        			"Error trying to open the following files in haploview " +
		        			"[ " + filename + "], [ " + infile + "]. \n" +
		        			"openHaplo:TreePopup.java");
		        }
			}
		
			OpenHapDialog.this.dispose();
			
		}
		
	};
	

	private void populateFields(String givenOp, String filename){
		for(int i = 0; i < pedSuffex.length; i ++){
			if(filename.endsWith(pedSuffex[i])){
				pedInfoRB.setSelected(true);
				pedFile.setText(filename);
				
				if(givenOp != null){
					String [] allInfiles = frame.data.getOpFiles(givenOp, Project.INFILE_KEY)[0];
					for(int k = 0; k < allInfiles.length; k++){
						if(allInfiles[k].endsWith(".info")){
							if(frame.data.isRemote(false)){
								infoFile.setText( new File(frame.data.getRoot(), 
										GeneralUtil.fileName(allInfiles[k])).getAbsolutePath());
							}else
								infoFile.setText( allInfiles[k]);
						}
						
					}
				}
			}
		}
		for(int i = 0; i < infoSuffex.length; i ++){
			if(filename.endsWith(infoSuffex[i])){
				pedInfoRB.setSelected(true);
				infoFile.setText(filename);
				
				String [] allInfiles = frame.data.getOpFiles(givenOp, Project.INFILE_KEY)[0];
				for(int k = 0; k < allInfiles.length; k++){
					if(allInfiles[k].endsWith(".ped")){
						if(frame.data.isRemote(false)){
							pedFile.setText( new File(frame.data.getRoot(), 
									GeneralUtil.fileName(allInfiles[k])).getAbsolutePath());
						}else
							pedFile.setText( allInfiles[k]);
					}
					
				}
			}				
		}
		for(int i = 0; i < nonSNPSuffex.length; i ++){
			if(filename.endsWith(nonSNPSuffex[i])){
				nonSNPRB.setSelected(true);
				nonSNPFile.setText(filename);
			}				
		}
		for(int i = 0; i < SNPSuffex.length; i ++){
			if(filename.endsWith(SNPSuffex[i])){
				SNPRB.setSelected(true);
				SNPFile.setText(filename);
				if(givenOp!=null){
					String [] allInfiles = frame.data.getOpFiles(givenOp, Project.INFILE_KEY)[0];
					for(int k = 0; k < allInfiles.length; k++){
						if(allInfiles[k].endsWith(".map")
								|| allInfiles[k].endsWith(".bim")){
							if(frame.data.isRemote(false)){
								mapFile.setText( new File(frame.data.getRoot(), 
										GeneralUtil.fileName(allInfiles[k])).getAbsolutePath());
							}else
								mapFile.setText( allInfiles[k]);
						}
						
					}
				}
			}				
		}
	}
	
	ItemListener setFields = new ItemListener(){
		public void itemStateChanged(ItemEvent arg0) {
			pedFile.setEditable(pedInfoRB.isSelected());
			pedButton.setEnabled(pedInfoRB.isSelected());
			infoFile.setEditable(pedInfoRB.isSelected());
			infoButton.setEnabled(pedInfoRB.isSelected());
			nonSNPFile.setEditable(nonSNPRB.isSelected());
			nonSNPButton.setEnabled(nonSNPRB.isSelected());
			SNPFile.setEditable(SNPRB.isSelected());
			SNPButton.setEnabled(SNPRB.isSelected());
			mapFile.setEditable(SNPRB.isSelected());
			mapButton.setEnabled(SNPRB.isSelected());
		}
	};
	
	void createPanel(){
		mapFile = new JTextField(20);
		SNPFile = new JTextField(20);
		nonSNPFile = new JTextField(20);
		pedFile = new JTextField(20);
		infoFile = new JTextField(20);
		
		pedButton  = new GeneralUtil.pickAFileButton(pedSuffex, "PED file", pedFile, frame);
		infoButton = new GeneralUtil.pickAFileButton(infoSuffex, "Info file", infoFile, frame);
		nonSNPButton = new GeneralUtil.pickAFileButton(nonSNPSuffex, "nonSNP file", nonSNPFile, frame);
		mapButton = new GeneralUtil.pickAFileButton(mapSuffex, "MAP file", mapFile, frame);
		SNPButton = new GeneralUtil.pickAFileButton(SNPSuffex, "SNP file", SNPFile, frame);
		
		pedInfoRB = new JRadioButton("ped/info file set");
		pedInfoRB.addItemListener(setFields);
		nonSNPRB = new JRadioButton("Non-SNP files");
		nonSNPRB.addItemListener(setFields);
		SNPRB = new JRadioButton("SNP files");
		SNPRB.addItemListener(setFields);
		
		ButtonGroup pickOne = new ButtonGroup();
		pickOne.add(pedInfoRB);
		pickOne.add(nonSNPRB);
		pickOne.add(SNPRB);
		
		ok = new JButton("OK");
		cancel = new JButton("Cancel");
		JPanel buttonPane = new JPanel();
		
		Container mainPane = this.getContentPane();
		this.getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		
		mainPane.add(pedInfoRB, c);
		
		c.gridx = 1;
		c.weightx = 1;
		mainPane.add(pedFile,c);
		
		c.gridx = 2;
		c.weightx = 0;
		mainPane.add(pedButton, c);
		
		c.gridx = 3;
		mainPane.add(new JLabel("(-info)"), c);
		
		c.gridx = 4;
		c.weightx = 1;
		mainPane.add(infoFile, c);
		
		c.gridx = 5;
		c.weightx = 0;
		mainPane.add(infoButton, c);
		
		c.gridy = 1;
		c.gridx = 0;
		c.weightx = 0;
		mainPane.add(nonSNPRB, c);
		
		c.gridx = 1;
		c.weightx = 1;
		mainPane.add(nonSNPFile,c);
		
		c.gridx = 2;
		c.weightx = 0;
		mainPane.add(nonSNPButton, c);
		
		c.gridy = 2;
		c.gridx = 0;
		c.weightx = 0;
		mainPane.add(SNPRB, c);
		
		c.gridx = 1;
		c.weightx = 1;
		mainPane.add(SNPFile,c);
		
		c.gridx = 2;
		c.weightx = 0;
		mainPane.add(SNPButton, c);
		
		c.gridx = 3;
		mainPane.add(new JLabel("(-map)"), c);
		
		c.gridx = 4;
		c.weightx = 1;
		mainPane.add(mapFile, c);
		
		c.gridx = 5;
		c.weightx = 0;
		mainPane.add(mapButton, c);
		
		c.gridy = 3;
		c.gridx = 0;
		c.weightx = 1;
		c.gridwidth = 6;
		mainPane.add(buttonPane, c);
		
		buttonPane.add(ok);
		ok.addActionListener(openHaplo);
		buttonPane.add(cancel);
		cancel.addActionListener(dispose);
	}
	
	public OpenHapDialog(MainFrame mf, 
			String givenOp, String givenFile){
		super(mf, "Open File in Haploview");
		frame = mf;
		createPanel();
		populateFields(givenOp, givenFile);
		pack();
		setVisible(true);
	}

	public OpenHapDialog(MainFrame mf, 
			String givenFile){
		super(mf, "Open File in Haploview");
		frame = mf;
	}
	
	/**
	 * Checks to see if a file can be opened in haploview
	 * @param filename string that has a file name
	 * @return true if the file name extention matches a haploview ok list,
	 * false othewise
	 */
	public static boolean hapoloviewOK(String filename){
		for(int i = 0; i < pedSuffex.length; i ++){
			if(filename.endsWith(pedSuffex[i]))
				return true;
		}
		for(int i = 0; i < infoSuffex.length; i ++){
			if(filename.endsWith(infoSuffex[i]))
				return true;
		}
		for(int i = 0; i < nonSNPSuffex.length; i ++){
			if(filename.endsWith(nonSNPSuffex[i]))
				return true;
		}
		for(int i = 0; i < SNPSuffex.length; i ++){
			if(filename.endsWith(SNPSuffex[i]))
				return true;
		}
		return false;
	}
}
